/* R-Dev-Ino Servo Controller (slave only)
   - uses I2C or Serial communication interfaces
   - uses SoftwareServo library
*/ 


// R-Dev-Ino board pinout:
//
// Ext  RDI Funct Arduino  ATmega168      Arduino Funct RDI External
//                       +-----\/----+
//          Reset       1| PC6   PC5 |28  D19 A5  SCL
//          Rx    D0    2| PD0   PC4 |27  D18 A4  SDA
//          Tx    D1    3| PD1   PC3 |26  D17 A3        
//      Wst Int0  D2    4| PD2   PC2 |25  D16 A2        
//      Scn Int1  D3    5| PD3   PC1 |24  D15 A1        
//      Lsh       D4    6| PD4   PC0 |23  D14 A0        
//                      7| VCC   GND |22  
//                      8| GND  AREF |21  
//          Xtal        9| PB6  AVCC |20  
//          Xtal       10| PB7   PB5 |19  D13      SCK  LED
//      Rsh OC0B  D5   11| PD5   PB4 |18  D12      MISO 
//      Lar OC0A  D6   12| PD6   PB3 |17  D11 OC2A MOSI       
//      Rar       D7   13| PD7   PB2 |16  D10 OC1B      
//      Lhd       D8   14| PB0   PB1 |15  D 9 OC1A      Rhd
//                       +-----------+
//

#include <Servo.h>
#include <EEPROM.h>
#include <Wire.h>

// I2C slaves addresses
#define Compass         0x21
#define Speech          50>>1
#define MotorController 60>>1
#define ServoController 70>>1
#define LedBar          80>>1
#define Mapper          90>>1
#define Brain           100>>1
#define Thermo          208>>1

// I2C commands
#define Forward     1 //MotorController
#define Reverse     2 //MotorController
#define Left        3 //MotorController/ServoController
#define Right       4 //MotorController/ServoController
#define StopAll     5 //MotorController
#define GoFwd       6 //MotorController
#define GoBwd       7 //MotorController
#define Err         8 //MotorController

#define Up          1 //ServoController
#define Down        2 //ServoController

#define Swaist     11 //ServoController
#define SleftSh    12 //ServoController
#define SrightSh   13 //ServoController
#define SleftArm   14 //ServoController
#define SrightArm  15 //ServoController
#define Sscan      16 //ServoController
#define SleftHand  17 //ServoController
#define SrightHand 18 //ServoController
#define SpanHead   19 //ServoController
#define StiltHead  20 //ServoController
#define StorePos   21 //ServoController
#define PickUpSO   22 //ServoController
#define ReleaseSO  23 //ServoController
#define Wave       24 //ServoController
#define PickUpBO   25 //ServoController
#define ReleaseBO  26 //ServoController

#define LCD1       31 //Speech
#define LCD2       32 //Speech
#define Heading    33 //Speech
#define Node       34 //Speech
#define Room       35 //Speech
#define Ping       36 //Speech
#define Sharp      37 //Speech
#define Angle      38 //Speech
#define Compass    39 //Speech
#define Temp       40 //Speech
#define Cluster    41 //Speech
#define Fire       42 //Speech
#define NoFire     43 //Speech

#define Scan       61 //Mapper
#define Fire       62 //Mapper
#define Fetch      63 //Mapper
#define Dance      64 //Mapper

#define Done       99 //Mapper

// Servo pins
#define WaistPin     2  // digital pin 2
#define LshoulderPin 3  // digital pin 3
#define RshoulderPin 4  // digital pin 4
#define ScannerPin   5  // digital pin 5
#define RarmPin      6  // digital pin 6
#define LarmPin      7  // digital pin 7
#define RhandPin     8  // digital pin 8
#define LhandPin     9  // digital pin 9
#define PanPin      10  // digital pin 10
#define TiltPin     11  // digital pin 11

// Servos
Servo Scanner;
Servo Waist;
Servo Lshoulder;
Servo Rshoulder;
Servo Larm;
Servo Rarm;
Servo Pan;
Servo Tilt;

//servos:    stepDeleay, Waist, Lshould, Rshould, Larm, Rarm, Lhand, Rhand
byte oldPos[8] = {0,      70,     180,     180,    80,   90,   80,    80};

// I2C
byte I2C[5]={0,0,0,0,0}; 
#define Address   0
#define Command   1
#define Value1    2
#define Value2    3

// Variables
byte Stand_By = 0;
byte Rarm_Wave = 1;
byte Pick_Up_SO = 2;
byte Release_SO = 3;
byte Pick_Up_BO = 4;
byte Release_BO = 5;
byte moveNumber = 0;
byte sv = 1;
byte pos = 80;
byte index = 1;
byte pos_delay = 0;
boolean move = 0;
boolean state = 0;

#define center  80
#define LedPin 13
//-----------------------------------------------------------------------------
void setup() { 
   
  Waist.attach(WaistPin);
  Scanner.attach(ScannerPin);
  Lshoulder.attach(LshoulderPin);
  Rshoulder.attach(RshoulderPin);
  Larm.attach(LarmPin);
  Rarm.attach(RarmPin);
  Pan.attach(PanPin);
  Tilt.attach(TiltPin);
  Center_servos();

  pinMode(LedPin, OUTPUT); 
  digitalWrite(LedPin,LOW); // LED OFF

  //Serial.begin (9600);
  //Serial.println("start");

  Wire.begin(ServoController);     // join i2c bus with address #70
  Wire.onReceive(I2CreceiveEvent); // register event
  //StoreMoves();
} 

//-----------------------------------------------------------------------------
void loop(){
  Process_commands();
        //Play_Move(Rarm_Wave); // Rarm_Wave
  //delay(1000);
        //Play_Move(Pick_Up_BO); // Pick_Up_Object
  //delay(1000);
        //Play_Move(Stand_By); // Pick_Up_Object
  //delay(1000);
       // Play_Move(Release_BO); // Release
 // delay(1000);
  //sweep();
  //SoftwareServo::refresh();
}  
//-----------------------------------------------------------------------------

void sweep() 
{ 
  Lshoulder.write(80);
  Rshoulder.write(80);
  StepDelay();
for(pos = 0; pos < 180; pos += 1)  // goes from 0 degrees to 180 degrees 
  {                                  // in steps of 1 degree 
    Larm.write(pos);              // tell servo to go to position in variable 'pos' 
    Rarm.write(pos);              // tell servo to go to position in variable 'pos' 
    //SoftwareServo::refresh();
    delay(15);
  } 
  for(pos = 180; pos>=1; pos-=1)     // goes from 180 degrees to 0 degrees 
  {                                
    Larm.write(pos);              // tell servo to go to position in variable 'pos' 
    Rarm.write(pos);              // tell servo to go to position in variable 'pos' 
    //SoftwareServo::refresh();
    delay(15);
  } 
  Larm.write(center+20);
  Rarm.write(center-20);
  StepDelay();
  Lshoulder.write(180);
  Rshoulder.write(180);
  StepDelay();
  StepDelay();
  StepDelay();
  StepDelay();
  delay(1000);
}



void Process_commands() {
  switch (I2C[Command]) {
      case Up:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Up");
        SwitchServo(I2C[Value1],Up);
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case Down:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Down");
        SwitchServo(I2C[Value1],Down);
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case Left:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Left");
        SwitchServo(I2C[Value1],Left);
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case Right:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Right");
        SwitchServo(I2C[Value1],Right);
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case Swaist:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Waist");
        Waist.write(I2C[Value1]); // Move Waist servo
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case SleftSh:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Left Shoulder");
        Lshoulder.write(I2C[Value1]); // Move L Shoulder servo
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case SrightSh:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Right Shoulder");
        Rshoulder.write(I2C[Value1]); // Move R Shoulder servo
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case SleftArm:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Left Arm");
        Larm.write(I2C[Value1]); // Move L Arm servo
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case SrightArm:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Right Arm");
        Rarm.write(I2C[Value1]); // Move R Arm servo
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case Sscan:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Scanner");
        Scanner.write(I2C[Value1]); // Move Scanner servo
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case SpanHead:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Pan");
        Pan.write(I2C[Value1]); // Move Pan servo
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case StiltHead:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Tilt");
        Tilt.write(I2C[Value1]); // Move Tilt servo
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case StorePos:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Store Position");
        StorePosition(I2C[Value1]); // 
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case PickUpSO:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Pick_Up_Object");
        Play_Move(Pick_Up_SO); // Pick_Up_Small_Object
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case ReleaseSO:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Release");
        Play_Move(Release_SO); // Release_Small_Object
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case Wave:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Wave");
        Play_Move(Rarm_Wave); // Rarm_Wave
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case PickUpBO:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Pick_Up_Object");
        Play_Move(Pick_Up_BO); // Pick_Up_Big_Object
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
      case ReleaseBO:
        digitalWrite(LedPin,HIGH); // LED ON
        //Serial.println("Release");
        Play_Move(Release_BO); // Release_Big_Object
        I2C[Command]=0;
        digitalWrite(LedPin,LOW); // LED OFF
        break;
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void Center_servos(){
  Waist.write(center-15);  // 65 = Up, 145 = Down
  Scanner.write(center+2); // 40 - 85 - 130 ; 40 = left, 130 = right
  Lshoulder.write(center+100); // 0 - 80 - 180
  Rshoulder.write(center+100); // 0 - 80 - 180 ; 0 = Up, 180 = Down
  Larm.write(center+5); // 0 - 80 - 180
  Rarm.write(center); // 0 - 80 - 180; 0 = 
  Pan.write(center); // 0 - 80 - 180
  Tilt.write(center-10); // 0 - 80 - 180; 0 = 
  //SoftwareServo::refresh();
  StepDelay();
}

void Play_Move(byte moveNumber) {
   //LCDposition(1, 1);
   //LcdSerial.print("Playing         ");
   //LCDposition(2, 1);
   //LcdSerial.print("Move ");
   //LcdSerial.print(move, DEC);
   //LcdSerial.print("          ");
  byte moveStart = EEPROM.read(490+(2*moveNumber));
  byte moveEnd = EEPROM.read(491+(2*moveNumber));
  for (byte i = moveStart; i <= moveEnd; i+=2){
    sv = EEPROM.read(i);
    pos = EEPROM.read(i+1); 
    switch (sv) {
      case 0:
        for (int u=1; u<=pos; u++){
          StepDelay();
        }
        break;
      case 1:
        Waist.write(pos);
        break;
      case 2:
        Lshoulder.write(pos);
        break;
      case 3:
        Rshoulder.write(pos);
        break;
      case 4:
        Larm.write(pos);
        break;
      case 5:
        Rarm.write(pos);
        break;
    }
  }
}

void StorePosition(byte servo){
  
}

void SwitchServo(byte servo,byte dir){
  byte pos=0;
  switch (servo){
    case Swaist:
      pos=Waist.read();
      if (dir==Up) pos=pos-5;
      if (dir==Down) pos=pos+5;
      Waist.write(pos);
      //Serial.print("Waist Servo ");
      break;
    case SleftSh:
      pos=Lshoulder.read();
      if (dir==Up) pos=pos+5;
      if (dir==Down) pos=pos-5;
      Lshoulder.write(pos);
      //Serial.print("L Shoulder Servo ");
      break;
    case SrightSh:
      pos=Rshoulder.read();
      if (dir==Up) pos=pos+5;
      if (dir==Down) pos=pos-5;
      Rshoulder.write(pos);
      //Serial.print("R Shoulder Servo ");
      break;
    case SleftArm:
      pos=Larm.read();
      if (dir==Left) pos=pos-5;
      if (dir==Right) pos=pos+5;
      Larm.write(pos);
      //Serial.print("L Arm Servo ");
      break;
    case SrightArm:
      pos=Rarm.read();
      if (dir==Left) pos=pos-5;
      if (dir==Right) pos=pos+5;
      Rarm.write(pos);
      //Serial.print("R Arm Servo ");
      break;
    case Sscan:
      pos=Scanner.read();
      if (dir==Left) pos=pos-5;
      if (dir==Right) pos=pos+5;
      Scanner.write(pos);
      //Serial.print("Scanner Servo ");
      break;
    case SpanHead:
      pos=Pan.read();
      if (dir==Left) pos=pos-5;
      if (dir==Right) pos=pos+5;
      Pan.write(pos);
      //Serial.print("Pan Servo ");
      break;
    case StiltHead:
      pos=Tilt.read();
      if (dir==Up) pos=pos-5;
      if (dir==Down) pos=pos+5;
      Tilt.write(pos);
      //Serial.print("Tilt Servo ");
      break;
  }
  //Serial.print("Position: ");
  //Serial.println(pos,DEC);
}


void StepDelay() {
    for (byte t=0; t<10; t++){
      //SoftwareServo::refresh();
      delay(20);
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void I2CreceiveEvent(int i)
{
    //Serial.println("I2C communication received:");
    while(0 < Wire.available()) // loop through all
    {
      I2C[i] = Wire.receive(); // receive bytes
      //Serial.print(i, DEC);
      //Serial.print("  ");
      //Serial.println(I2C[i], DEC);
      i = i-1;
    }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


